"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Dict_1 = __importDefault(require("./Dict"));
const filter_1 = __importDefault(require("./filter"));
class Counter {
    map;
    constructor(init) {
        this.map = new Dict_1.default(init);
    }
    static count(elems, key) {
        const counter = new Counter();
        for (const elem of elems) {
            counter.increment(key(elem));
        }
        return counter;
    }
    adjust(val, diff) {
        const newCount = this.map.getOrDefault(val, 0) + diff;
        this.map.put(val, newCount);
        return newCount;
    }
    entries() {
        return this.map.entries();
    }
    positiveEntries() {
        return (0, filter_1.default)(this.map.entries(), ([_, count]) => count > 0);
    }
    values() {
        return this.map.keys();
    }
    counts() {
        return this.map.values();
    }
    increment(val) {
        return this.adjust(val, 1);
    }
    decrement(val) {
        return this.adjust(val, -1);
    }
    get(val) {
        return this.map.getOrDefault(val, 0);
    }
    set(val, count) {
        this.map.put(val, count);
        return this;
    }
    [Symbol.iterator]() {
        return this.map[Symbol.iterator]();
    }
}
exports.default = Counter;
