type Tasks = {
    [name: string]: {
        input: any;
        output: any;
    };
};
type Ctx<AllTasks extends Tasks, State> = {
    state: State;
    execute: <T extends keyof AllTasks & string>(t: T, i: AllTasks[T]["input"]) => Promise<AllTasks[T]["output"]>;
};
export default class WorkerPool<LeaderTasks extends Tasks = {}, WorkerTasks extends Tasks = {}, LeaderState = void, WorkerState = void> {
    private readonly scriptPath;
    private readonly n;
    private stateInit?;
    private entrypoint?;
    private taskHandlers;
    constructor(scriptPath: string, n?: number);
    leader(fn: (ctx: Ctx<WorkerTasks, LeaderState>) => Promise<unknown>): this;
    worker(fn: (ctx: Ctx<LeaderTasks, WorkerState>) => Promise<unknown>): this;
    leaderState<S>(fn: () => Promise<S>): WorkerPool<LeaderTasks, WorkerTasks, S, WorkerState>;
    workerState<S>(fn: () => Promise<S>): WorkerPool<LeaderTasks, WorkerTasks, LeaderState, S>;
    leaderTask<T extends string, I, O>(name: T, handler: (i: I, ctx: Ctx<WorkerTasks, LeaderState>) => Promise<O>): WorkerPool<LeaderTasks & {
        [name in T]: {
            input: I;
            output: O;
        };
    }, WorkerTasks, LeaderState, WorkerState>;
    workerTask<T extends string, I, O>(name: T, handler: (i: I, ctx: Ctx<LeaderTasks, WorkerState>) => Promise<O>): WorkerPool<LeaderTasks, WorkerTasks & {
        [name in T]: {
            input: I;
            output: O;
        };
    }, LeaderState, WorkerState>;
    go(): Promise<void>;
}
export {};
